@@ -89,9 +89,12 @@ class Advertisement(object):
     
     def unpublish(self):
         """End the advertisement."""
-        self.group.Reset()
-        self.logging.info("ending advertising %s on port %d",
-                self.stype, self.port)
+        try:
+            self.logging.info("ending advertising %s on port %d",
+                    self.stype, self.port)
+            self.group.Reset()
+        except DBusError, e:
+            self.logging.warn("Couldn't reset DBus at shutdown. %s" % (e,))
         self.group = None
 
     def die(self):
