@@ -49,9 +49,17 @@ class CouchDatabase(server_base.CouchDatabaseBase):
     def __init__(self, database, uri=None, record_factory=None, create=False,
                  server_class=OAuthCapableServer, oauth_tokens=None,
                  ctx=desktopcouch.local_files.DEFAULT_CONTEXT):
-        if not uri:
-            port = desktopcouch.find_port(ctx=ctx)
-            uri = "http://localhost:%s" % port
+        self.ctx = ctx
+        self.server_uri = uri
         super(CouchDatabase, self).__init__(
                 database, uri, record_factory=record_factory, create=create,
                 server_class=server_class, oauth_tokens=oauth_tokens, ctx=ctx)
+
+    def _reconnect(self):
+        if not self.server_uri:
+            port = desktopcouch.find_port(ctx=self.ctx)
+            uri = "http://localhost:%s" % port
+        else:
+            uri = self.server_uri
+        super(CouchDatabase, self)._reconnect(uri=uri)
+
