@@ -22,7 +22,7 @@
 """The Desktop Couch Records API."""
 
 import httplib2, urlparse, cgi, copy
-from time import time
+from time import time, sleep
 import socket
 
 # please keep desktopcouch python 2.5 compatible for now
@@ -150,42 +150,51 @@ class CouchDatabaseBase(object):
         self._server_class_extras = server_class_extras
         self.record_factory = record_factory or Record
         self.server_uri = uri
+        self._server = None
+        self.db = None
         self._reconnect()
         self._changes_since = self.db.info()["update_seq"]
         self._changes_last_used = 0  # Immediate run works.
 
     @staticmethod
-    def _is_bug_lp539674(ex):
+    def _is_reconnection_fail(ex):
         return isinstance(ex, AttributeError) and \
                 ex.args == ("'NoneType' object has no attribute 'makefile'",)
 
     def with_reconnects(self, func, *args, **kwargs):
-        for retry in (2, 1, None):
+        for retry in (3, 2, 1, None):
             try:
                 return func(*args, **kwargs)
             except Exception, e:
-                if self._is_bug_lp539674(e) and retry:
-                    logging.warn("DB connection timed out.  Reconnecting.")
+                if self._is_reconnection_fail(e) and retry:
+                    logging.warn("DB connection failed.  Reconnecting.")
                     self._reconnect()
                     continue
                 elif isinstance(e, socket.error):
                     logging.warn("Other socket error %s.  Reconnecting.", e)
-                    time.sleep(0.3)  
+                    sleep(0.3)  
                     self._reconnect()
                     continue
                 else:
                     raise
+        raise ValueError("failed to (re-)connect to couchdb server")
 
-    def _reconnect(self):
-        logging.info("Connecting to %s." % (self.server_uri or "discovered local port",))
-        self._server = self._server_class(self.server_uri,
+    def _reconnect(self, uri=None):
+        logging.info("Connecting to %s.", self.server_uri or "discovered local port")
+
+        self._server = self._server_class(uri or self.server_uri,
                 **self._server_class_extras)
         if self._database_name not in self._server:
             if self._create:
                 self._server.create(self._database_name)
             else:
                 raise NoSuchDatabase(self._database_name)
-        self.db = self._server[self._database_name]
+        if self.db is None:
+            self.db = self._server[self._database_name]
+        else:
+            # Monkey-patch the object the user already uses.  Oook!
+            new_db = self._server[self._database_name]
+            self.db.resource = new_db.resource
 
     def _temporary_query(self, map_fun, reduce_fun=None, language='javascript',
             wrapper=None, **options):
@@ -225,8 +234,26 @@ class CouchDatabaseBase(object):
             # Do not rely on couchdb to create an ID for us.
             from uuid import uuid4
             record.record_id = uuid4().hex
-        self.with_reconnects(self.db.__setitem__,
-                record.record_id, record._data)
+        try:
+            self.with_reconnects(self.db.__setitem__,
+                    record.record_id, record._data)
+        except ResourceConflict:
+            if record.record_revision is None and not row_is_deleted(record):
+                old_record = self.with_reconnects(self.db.__getitem__,
+                    record.record_id)
+                if row_is_deleted(old_record):
+                    # User asked to make new record that already exists
+                    # but we have marked deleted internally.  Instead of
+                    # complaining, pull up the previous revision ID and
+                    # add that to the user's record, and re-send it.
+                    record._set_record_revision(old_record.rev)
+
+                    self.with_reconnects(self.db.__setitem__,
+                            record.record_id, record._data)
+                else:
+                    raise
+            else:
+                raise
 
         for attachment_name in getattr(record, "_detached", []):
             self.with_reconnects(self.db.delete_attachment,
@@ -362,9 +389,10 @@ class CouchDatabaseBase(object):
 
     def record_exists(self, record_id):
         """Check if record with given id exists."""
-        if record_id not in self.db:
+        try:
+            record = self.with_reconnects(self.db.__getitem__, record_id)
+        except ResourceNotFound:
             return False
-        record = self.with_reconnects(self.db.__getitem__, record_id)
         return not row_is_deleted(record)
 
     def delete_view(self, view_name, design_doc=DEFAULT_DESIGN_DOCUMENT):
@@ -414,12 +442,79 @@ class CouchDatabaseBase(object):
     def execute_view(self, view_name, design_doc=DEFAULT_DESIGN_DOCUMENT,
             **params):
         """Execute view and return results."""
+
+        class ReconnectingViewWrapper(object):
+            """A view from python-couchdb is an object with attributes that
+            cause HTTP requests to be fired off when accessed.  If we wish
+            to be able to reconnect on disappearance of the server, then
+            we must intercept calls from user to python-couchdb."""
+
+            def __init__(wrapper, obj, *args, **kwargs):
+                wrapper.obj = self.with_reconnects(obj, *args, **kwargs)
+
+            def ___call__(wrapper, **options):
+                return self.with_reconnects(wrapper.obj.__call__, **options)
+
+            def __iter__(wrapper):
+                return self.view_with_reconnects(wrapper.obj.__iter__,
+                                                 wrapper.obj)
+
+            def __len__(wrapper):
+                return self.view_with_reconnects(wrapper.obj.__len__,
+                                                 wrapper.obj)
+
+            def __getattr__(wrapper, key):
+                return self.with_reconnects(getattr, wrapper.obj, key)
+
+            def __getitem__(wrapper, key):
+                return ReconnectingViewWrapper(wrapper.obj.__getitem__, key)
+
         if design_doc is None:
             design_doc = view_name
 
         view_id_fmt = "_design/%(design_doc)s/_view/%(view_name)s"
-        return self.with_reconnects(self.db.view, view_id_fmt % locals(),
-                **params)
+        wrapper = ReconnectingViewWrapper(self.db.view, 
+                                          view_id_fmt % locals(),
+                                          **params)
+        return wrapper
+
+    def view_with_reconnects(self, func, view, *args, **kwargs):
+        for retry in (3, 2, 1, None):
+            try:
+                return func(*args, **kwargs)
+            except Exception, e:
+                if self._is_reconnection_fail(e) and retry:
+                    logging.warn("DB connection failed.  Reconnecting.")
+                    self._reconnect_view(view_result=view)
+                    continue
+                elif isinstance(e, socket.error):
+                    logging.warn("Other socket error %s.  Reconnecting.", e)
+                    sleep(0.3)  
+                    self._reconnect()
+                    continue
+                else:
+                    raise
+        raise ValueError("failed to (re-)connect to couchdb server")
+
+    def _reconnect_view(self, uri=None, view_result=None):
+        logging.info("Connecting to %s.",
+                     self.server_uri or "discovered local port")
+        self._server = self._server_class(
+            uri or '/'.join(self.db.resource.uri.split('/')[:-1]),
+                **self._server_class_extras)
+        if self._database_name not in self._server:
+            if self._create:
+                self._server.create(self._database_name)
+            else:
+                raise NoSuchDatabase(self._database_name)
+        if self.db is None:
+            self.db = self._server[self._database_name]
+        else:
+            # Monkey-patch the object the user already uses.  Oook!
+            view_url = urlparse.urlparse(view_result.view.resource.uri)
+            db_url = urlparse.urlparse(self.db.resource.uri)
+            view_result.view.resource.uri = urlparse.urlunparse(
+                [view_url[0], db_url[1]] + list(view_url[2:]))
 
     def add_view(self, view_name, map_js, reduce_js,
             design_doc=DEFAULT_DESIGN_DOCUMENT):
@@ -429,7 +524,7 @@ class CouchDatabaseBase(object):
             design_doc = view_name
 
         view = ViewDefinition(design_doc, view_name, map_js, reduce_js)
-        view.sync(self.db)
+        self.with_reconnects(view.sync, self.db)
         assert self.view_exists(view_name, design_doc)
 
     def view_exists(self, view_name, design_doc=DEFAULT_DESIGN_DOCUMENT):
@@ -555,6 +650,7 @@ class CouchDatabaseBase(object):
             uri = couchdburi(
                 self._server.resource.uri, self.db.name, "_changes",
                 since=self._changes_since)
+            ## Assume server has not crashed and URI is the same.  FIXME
             resp, data = self._server.resource.http.request(uri, "GET", "", {})
             if resp["status"] != '200':
                 raise IOError(
@@ -571,3 +667,12 @@ class CouchDatabaseBase(object):
             # If exception in cb, we never update governor.
             self._changes_last_used = now
         return call_count
+
+    def ensure_full_commit(self):
+        """
+        Make sure that CouchDb flushes all writes to the database,
+        flushing all delayed commits, before going on.
+        """
+        self.db.resource.post(
+            path='_ensure_full_commit',
+            headers={'Content-Type': 'application/json'})
