@@ -19,12 +19,17 @@
 
 """testing database/contact.py module"""
 import testtools
+import os
+import signal
+import time
 
 import desktopcouch.tests as test_environment
 from desktopcouch.records.server import CouchDatabase
 from desktopcouch.records.server_base import (
-    row_is_deleted, NoSuchDatabase, FieldsConflict)
+    row_is_deleted, NoSuchDatabase, FieldsConflict, ResourceConflict)
 from desktopcouch.records.record import Record
+from desktopcouch.stop_local_couchdb import stop_couchdb
+from desktopcouch import find_pid
 
 # pylint can't deal with failing imports even when they're handled
 # pylint: disable-msg=F0401
@@ -53,7 +58,7 @@ class TestCouchDatabase(testtools.TestCase):
         # Connect to CouchDB server
         self.dbname = self._testMethodName
         self.database = CouchDatabase(self.dbname, create=True,
-                ctx=test_environment.test_context)
+                ctx=self.get_test_context())
         #create some records to pull out and test
         self.database.put_record(Record({
             "key1_1": "val1_1", "key1_2": "val1_2", "key1_3": "val1_3",
@@ -67,8 +72,28 @@ class TestCouchDatabase(testtools.TestCase):
 
     def tearDown(self):
         """tear down each test"""
+        this_context = self.get_test_context()
+        if this_context != test_environment.test_context:
+            stop_couchdb(ctx=this_context)
         super(TestCouchDatabase, self).tearDown()
-        del self.database._server[self.dbname]
+
+    def get_test_context(self):
+        return test_environment.test_context
+
+    def maybe_die(self):
+        pass
+
+    def wait_until_server_dead(self, pid=None):
+        if pid is not None:
+            pid = find_pid(start_if_not_running=False, ctx=self.get_test_context())
+        if pid is None:
+            return
+        while True:
+            try:
+                os.kill(pid, 0)  # Wait until exited
+                time.sleep(0.1)
+            except OSError:
+                break
 
     def test_database_not_exists(self):
         self.assertRaises(
@@ -78,12 +103,14 @@ class TestCouchDatabase(testtools.TestCase):
         """Test getting mutliple records by type"""
         records = self.database.get_records(
             record_type="test.com",create_view=True)
-        self.assertEqual(3,len(records))
+        self.maybe_die()  # should be able to survive couchdb death
+        self.assertEqual(3, len(records))
 
     def test_get_record(self):
         """Test getting a record."""
         record = Record({'record_number': 0}, record_type="http://example.com/")
         record_id = self.database.put_record(record)
+        self.maybe_die()  # should be able to survive couchdb death
         retrieved_record = self.database.get_record(record_id)
         self.assertEqual(0, retrieved_record['record_number'])
 
@@ -91,7 +118,8 @@ class TestCouchDatabase(testtools.TestCase):
         """Test putting a record."""
         record = Record({'record_number': 0}, record_type="http://example.com/")
         record_id = self.database.put_record(record)
-        retrieved_record = self.database._server[self.dbname][record_id]
+        self.maybe_die()  # should be able to survive couchdb death
+        retrieved_record = self.database.get_record(record_id)
         self.assertEqual(
             record['record_number'], retrieved_record['record_number'])
 
@@ -121,15 +149,42 @@ class TestCouchDatabase(testtools.TestCase):
         record = Record({'record_number': 0}, record_type="http://example.com/")
         record_id = self.database.put_record(record)
         self.database.delete_record(record_id)
+        ###self.maybe_die()  # should be able to survive couchdb death
         deleted_record = self.database._server[self.dbname][record_id]
+        #deleted_record = self.database.get_record(record_id)
         self.assert_(deleted_record['application_annotations']['Ubuntu One'][
             'private_application_annotations']['deleted'])
 
+    def test_delete_and_recreate_record(self):
+        """Test deletion of records."""
+        old_record = Record({'record_number': 0},
+                record_type="http://example.com/", record_id="Reuse me")
+        old_record_id = self.database.put_record(old_record)
+        self.database.delete_record(old_record_id)
+
+        new_record = Record({'something else': 42},
+                record_type="http://example.com/", record_id=old_record_id)
+        new_record_id = self.database.put_record(new_record)
+
+        self.assertEqual(old_record_id, new_record_id)
+
+        new_record_fetched = self.database.get_record(new_record_id)
+        self.assertEqual(new_record_fetched["something else"], 42)
+
+        # this still fails, to be sure.
+        bad_record = Record({'again': 'prev not deleted'},
+                record_type="http://example.com/", record_id=old_record_id)
+        self.assertRaises(ResourceConflict,
+                self.database.put_record, bad_record)
+
+        self.database.delete_record(new_record_id)
+
     def test_get_deleted_record(self):
         """Test (not) getting a deleted record."""
         record = Record({'record_number': 0}, record_type="http://example.com/")
         record_id = self.database.put_record(record)
         self.database.delete_record(record_id)
+        self.maybe_die()  # should be able to survive couchdb death
         retrieved_record = self.database.get_record(record_id)
         self.assertEqual(None, retrieved_record)
 
@@ -138,6 +193,7 @@ class TestCouchDatabase(testtools.TestCase):
         record = Record({'record_number': 0}, record_type="http://example.com/")
         self.assert_(not self.database.record_exists("ThisMustNotExist"))
         record_id = self.database.put_record(record)
+        self.maybe_die()  # should be able to survive couchdb death
         self.assert_(self.database.record_exists(record_id))
         self.database.delete_record(record_id)
         self.assert_(not self.database.record_exists(record_id))
@@ -147,12 +203,14 @@ class TestCouchDatabase(testtools.TestCase):
         dictionary = {'record_number': 0, 'field1': 1, 'field2': 2}
         record = Record(dictionary, record_type="http://example.com/")
         record_id = self.database.put_record(record)
+        self.maybe_die()  # should be able to survive couchdb death
         # manipulate the database 'out of view'
         non_working_copy = self.database.get_record(record_id)
         non_working_copy['field2'] = 22
         non_working_copy['field3'] = 3
         self.database.put_record(non_working_copy)
         self.database.update_fields(record_id, {'field1': 11})
+        self.maybe_die()  # should be able to survive couchdb death
         working_copy = self.database.get_record(record_id)
         self.assertEqual(0, working_copy['record_number'])
         self.assertEqual(11, working_copy['field1'])
@@ -176,10 +234,12 @@ class TestCouchDatabase(testtools.TestCase):
         self.assertRaises(
             KeyError, self.database.delete_view, view2_name, design_doc)
         self.database.add_view(view1_name, map_js, reduce_js, design_doc)
+        self.maybe_die()  # should be able to survive couchdb death
         self.database.add_view(view2_name, map_js, reduce_js, design_doc)
         self.database.delete_view(view1_name, design_doc)
         self.assertRaises(
             KeyError, self.database.delete_view, view1_name, design_doc)
+        self.maybe_die()  # should be able to survive couchdb death
         self.database.delete_view(view2_name, design_doc)
         self.assertRaises(
             KeyError, self.database.delete_view, view2_name, design_doc)
@@ -208,11 +268,15 @@ class TestCouchDatabase(testtools.TestCase):
 
         results = self.database.get_records(create_view=True)
 
+        self.assertTrue(8 <= len(results.rows))  # our 8, plus callers' data
+        self.assertIn("record_type", results.rows[0].value)
+
         for row in results[good_record_type]:  # index notation
             self.assertTrue(row.id in record_ids_we_care_about)
             record_ids_we_care_about.remove(row.id)
             self.assertFalse(row_is_deleted(row))
 
+        self.maybe_die()  # should be able to survive couchdb death
         self.assertTrue(len(record_ids_we_care_about) == 0, "expected zero")
 
         self.assertRaises(KeyError, self.database.get_records,
@@ -226,6 +290,7 @@ class TestCouchDatabase(testtools.TestCase):
         map_js = """function(doc) { emit(doc._id, null) }"""
         self.database.add_view(view_name, map_js, None, design_doc)
 
+        self.maybe_die()  # should be able to survive couchdb death
         self.assertEqual(self.database.list_views(design_doc), [view_name])
         self.database.delete_view(view_name, design_doc)
 
@@ -233,11 +298,13 @@ class TestCouchDatabase(testtools.TestCase):
 
     def test_get_view_by_type_new_but_already(self):
         self.database.get_records(create_view=True)
+        self.maybe_die()  # should be able to survive couchdb death
         self.database.get_records(create_view=True)
         # No exceptions on second run?  Yay.
 
     def test_get_view_by_type_createxcl_fail(self):
         self.database.get_records(create_view=True)
+        self.maybe_die()  # should be able to survive couchdb death
         self.assertRaises(KeyError, self.database.get_records, create_view=None)
 
     def test_get_changes(self):
@@ -297,6 +364,8 @@ class TestCouchDatabase(testtools.TestCase):
         # Ensure time is same.
         self.assertEqual(saved_time, self.database._changes_last_used)
 
+        ###self.maybe_die()  # should be able to survive couchdb death
+
         # Next time we run, we get the same event again.
         # Consume queued changes.
         count = self.database.report_changes(rep)
@@ -305,6 +374,19 @@ class TestCouchDatabase(testtools.TestCase):
         # Ensure position different.
         self.assertEqual(saved_position + 1, self.database._changes_since)
 
+    def test_view_wrapper_has_all_attributes(self):
+        design_doc = "test_view_wrapper_has_all_attributes"
+        view1_name = "unit_tests_are_great_yeah"
+
+        map_js = """function(doc) { emit(doc._id, null) }"""
+
+        self.database.add_view(view1_name, map_js, None, design_doc)
+        view = self.database.execute_view(view1_name, design_doc)
+        
+        for attr_name in dir(view.obj):
+            # This works because hasattr actually calls getattr.
+            self.assertTrue(hasattr(view, attr_name))
+
     def test_report_changes_all_ops_give_known_keys(self):
         def rep(**kwargs):
             self.failUnless("changes" in kwargs)
@@ -346,6 +428,7 @@ class TestCouchDatabase(testtools.TestCase):
         constructed_record.attach(content, "nu/mbe/rs", "text/plain")
         constructed_record.attach("string", "another document", "text/plain")
 
+        ###self.maybe_die()  # should be able to survive couchdb death
         constructed_record.attach("XXXXXXXXX", "never used", "text/plain")
         constructed_record.detach("never used")  # detach works before commit.
 
@@ -358,6 +441,7 @@ class TestCouchDatabase(testtools.TestCase):
         self.assertRaises(KeyError, constructed_record.attach, content,
                 "another document", "text/x-rst")
 
+        ###self.maybe_die()  # should be able to survive couchdb death
         record_id = self.database.put_record(constructed_record)
         retrieved_record = self.database.get_record(record_id)
 
@@ -378,6 +462,7 @@ class TestCouchDatabase(testtools.TestCase):
         # get new
         retrieved_record = self.database.get_record(record_id)
 
+        ###self.maybe_die()  # should be able to survive couchdb death
         # We can get a list of attachments.
         self.assertEqual(set(retrieved_record.list_attachments()),
                 set(["nu/mbe/rs", "Document"]))
@@ -400,6 +485,7 @@ class TestCouchDatabase(testtools.TestCase):
         self.assertEqual(out_data, content.getvalue())
         self.assertEqual(out_content_type, "text/plain")
 
+        ###self.maybe_die()  # should be able to survive couchdb death
         # Asking for a named document that does not exist causes KeyError.
         self.assertRaises(KeyError, retrieved_record.attachment_data,
                 "NoExist")
@@ -430,6 +516,7 @@ class TestCouchDatabase(testtools.TestCase):
         # ordinary requests are in key order
         self.assertEqual(data, sorted(data))
 
+        self.maybe_die()  # should be able to survive couchdb death
         # now request descending order and confirm that it *is* descending
         descdata = [i.key for i in
             list(self.database.execute_view(view1_name, design_doc,
@@ -489,3 +576,38 @@ class TestCouchDatabase(testtools.TestCase):
             self.fail()
         except FieldsConflict, e:
             self.assertEqual({('field1',): (22, 11)}, e.conflicts)
+
+
+class TestServerDiesSegv(TestCouchDatabase):
+    def get_test_context(self):
+        try:
+            return self.ctx
+        except AttributeError:
+            self.ctx = test_environment.create_new_test_environment()
+            return self.ctx
+
+    def maybe_die(self):
+        self.database.ensure_full_commit()
+        time.sleep(2)
+        pid = find_pid(start_if_not_running=False, ctx=self.get_test_context())
+        if pid is None:
+            print "couchdb has already quit!  That's unexpected."
+            return
+        print "DIE, process", pid, "!"
+        os.kill(pid, signal.SIGSEGV)
+        self.wait_until_server_dead(pid=pid)
+
+
+class TestServerDiesNormal(TestCouchDatabase):
+    def get_test_context(self):
+        try:
+            return self.ctx
+        except AttributeError:
+            self.ctx = test_environment.create_new_test_environment()
+            return self.ctx
+
+    def maybe_die(self):
+        self.database.ensure_full_commit()
+        time.sleep(2)
+        stop_couchdb(ctx=self.get_test_context())
+        self.wait_until_server_dead()
