@@ -1,7 +1,9 @@
 from oauth import oauth
 import logging
 import httplib2
-import gnomekeyring
+
+import dbus
+import ubuntu_sso
 
 try:
     # Python 2.5
@@ -14,8 +16,6 @@ except ImportError:
 name = "Ubuntu One"
 description = "The Ubuntu One cloud service"
 
-oauth_consumer_key = "ubuntuone"
-oauth_consumer_secret = "hammertime"
 
 def in_main_thread(function, *args, **kwargs):
     from twisted.python.threadable import isInIOThread
@@ -37,50 +37,17 @@ def get_oauth_data():
     if oauth_data is not None:
         return oauth_data
 
-    try:
-        matches = in_main_thread(gnomekeyring.find_items_sync,
-            gnomekeyring.ITEM_GENERIC_SECRET,
-            {'ubuntuone-realm': "https://ubuntuone.com",
-             'oauth-consumer-key': oauth_consumer_key})
-        if matches:
-            # parse "a=b&c=d" to {"a":"b","c":"d"}
-            kv_list = [x.split("=", 1) for x in matches[0].secret.split("&")]
-            keys, values = zip(*kv_list)
-            keys = [k.replace("oauth_", "") for k in keys]
-            oauth_data = dict(zip(keys, values))
-            oauth_data.update({
-                "consumer_key": oauth_consumer_key,
-                "consumer_secret": oauth_consumer_secret,
-            })
-            return oauth_data
-    except ImportError, e:
-        logging.info("Can't replicate to Ubuntu One cloud without credentials."
-                " %s", e)
-    except gnomekeyring.NoMatchError:
-        logging.info("This machine hasn't authorized itself to Ubuntu One; "
-                "replication to the cloud isn't possible until it has.  See "
-                "'ubuntuone-client-applet'.")
-    except gnomekeyring.NoKeyringDaemonError:
-        logging.error("No keyring daemon found in this session, so we have "
-                "no access to Ubuntu One data.")
-    except gnomekeyring.CancelledError:
-        logging.error("User cancelled the keyring dialog. We have no access.")
+    bus = dbus.SessionBus()
+    proxy = bus.get_object(ubuntu_sso.DBUS_BUS_NAME,
+                           ubuntu_sso.DBUS_CRED_PATH,
+                           follow_name_owner_changes=True)
+    logging.info('get_oauth_data: asking for credentials to Ubuntu SSO. '
+                 'App name: %s', name)
+    oauth_data = proxy.find_credentials(name)
+    logging.info('Got credentials from Ubuntu SSO. Non emtpy credentials? %s',
+                 len(oauth_data) > 0)
+    return oauth_data
 
-def get_oauth_token(consumer):
-    """Get the token from the keyring"""
-    try:
-        items = in_main_thread(gnomekeyring.find_items_sync,
-            gnomekeyring.ITEM_GENERIC_SECRET,
-            {'ubuntuone-realm': "https://one.ubuntu.com",
-             'oauth-consumer-key': consumer.key})
-    except gnomekeyring.NoMatchError:
-        logging.info("No o.u.c key.  Maybe there's uo.c key?")
-        items = in_main_thread(gnomekeyring.find_items_sync,
-            gnomekeyring.ITEM_GENERIC_SECRET,
-            {'ubuntuone-realm': "https://ubuntuone.com",
-             'oauth-consumer-key': consumer.key})
-    if len(items):
-        return oauth.OAuthToken.from_string(items[0].secret)
 
 def get_oauth_request_header(consumer, access_token, http_url):
     """Get an oauth request header given the token and the url"""
@@ -107,17 +74,17 @@ class PrefixGetter():
 
         url = "https://one.ubuntu.com/api/account/"
         if self.oauth_header is None:
-            consumer = oauth.OAuthConsumer(oauth_consumer_key,
-                    oauth_consumer_secret)
             try:
-                access_token = get_oauth_token(consumer)
-            except gnomekeyring.NoKeyringDaemonError:
-                logging.info("No keyring daemon is running for this session.")
-                raise ValueError("No keyring access")
-            if not access_token:
-                logging.info("Could not get access token from keyring")
-                raise ValueError("No keyring access")
-            self.oauth_header = get_oauth_request_header(
+                get_oauth_data()
+                consumer = oauth.OAuthConsumer(oauth_data['consumer_key'],
+                                               oauth_data['consumer_secret'])
+                access_token = oauth.OAuthToken(oauth_data['token'],
+                                                oauth_data['token_secret'])
+            except:
+                logging.exception("Could not get access token from sso.")
+                raise ValueError("No access token.")
+
+        self.oauth_header = get_oauth_request_header(
                 consumer, access_token, url)
 
         client = httplib2.Http()
