@@ -22,9 +22,11 @@ Start local CouchDB server.
 Steps:
     1. Work out which folders to use (running from source tree, or installed)
     2. Actually start CouchDB
-    3. check design documents in central folder against design documents in
+    3. Check design documents in central folder against design documents in
        Couch, and overwrite any new ones
-    4. Write an HTML "bookmark" file which directs people to the local
+    4. Activate the pairing of ubuntu one by retrieving the oauth tokens using
+       ubuntu sso.
+    5. Write an HTML "bookmark" file which directs people to the local
        running CouchDB.
 
 This script is normally called by advertisePort.py, which advertises the local
@@ -33,66 +35,26 @@ activation.
 """
 
 from __future__ import with_statement
-import os, subprocess, sys, glob
-import desktopcouch
-from desktopcouch import local_files
-import xdg.BaseDirectory
+
 import errno
-import time
-from desktopcouch.records.server import CouchDatabase
-import logging
+import glob
 import itertools
-import re
-
-
-def process_is_couchdb__linux(pid):
-    if pid is None:
-        return False
-    pid = int(pid)  # ensure it's a number
-    proc_dir = "/proc/%d" % (pid,)
-    try:
-        # check to make sure it is actually a desktop-couch instance
-        with open(os.path.join(proc_dir, 'cmdline')) as cmd_file:
-            cmd = cmd_file.read()
-        #if '/desktop-couch' not in cmd:
-        if 'beam' not in cmd:
-            return False
-
-        # make sure it's our process.
-        if not os.access(os.path.join(proc_dir, "mem"), os.W_OK):
-            return False
-
-    except IOError:
-        return False
-
-    return True
-
+import logging
+import os
 import platform
-os_name = platform.system()
-try:
-    process_is_couchdb = {
-        "Linux": process_is_couchdb__linux
-        } [os_name]
-except KeyError:
-    raise NotImplementedError("os %r is not yet supported" % (os_name,))
+import re 
+import subprocess
+import sys
+import time
 
-def read_pidfile(ctx=local_files.DEFAULT_CONTEXT):
-    try:
-        pid_file = ctx.file_pid
-        if not os.path.exists(pid_file):
-            return None
-        with open(pid_file) as fp:
-            try:
-                contents = fp.read()
-                if contents == "\n":
-                    return None  # not yet written to pid file
-                return int(contents)
-            except ValueError:
-                logging.warn("Pid file does not contain int: %r", contents)
-                return None
-    except IOError, e:
-        logging.warn("Reading pid file caused error.  %s", e)
-        return None
+from gnomekeyring import NoKeyringDaemonError
+
+import desktopcouch
+from desktopcouch import local_files, read_pidfile, process_is_couchdb
+from desktopcouch.records.server import CouchDatabase
+from desktopcouch.pair.couchdb_pairing.ubuntuone_pairing import (
+    pair_with_ubuntuone)
+import xdg.BaseDirectory
 
 def run_couchdb(ctx=local_files.DEFAULT_CONTEXT):
     """Actually start the CouchDB process.  Return its PID."""
@@ -242,6 +204,29 @@ def update_bookmark_file(port, ctx=local_files.DEFAULT_CONTEXT):
     finally:
         fp.close()
 
+
+def update_pairing_service(sso=None, test_import_fails=False):
+    """Update the pairing records between desktopcouch and Ubuntu One."""
+    if not sso:
+        try:
+            # get the credentials using ubuntu sso dbus
+            if test_import_fails:
+                raise ImportError
+            from ubuntu_sso.main import SSOCredentials
+            sso = SSOCredentials()
+        except ImportError:
+            logging.info("Ubuntu SSO dbus service could not be used.")
+
+    if sso: 
+        try:
+            credentials = sso.find_credentials('Ubuntu One')
+        except NoKeyringDaemonError:
+            logging.info(
+                "NoKeyringDaemonError exception raised.")
+            credentials = None
+        if credentials and len(credentials) > 0:
+            pair_with_ubuntuone()
+ 
 def start_couchdb(ctx=local_files.DEFAULT_CONTEXT):
     """Execute each step to start a desktop CouchDB."""
     pid = None
@@ -259,7 +244,8 @@ def start_couchdb(ctx=local_files.DEFAULT_CONTEXT):
     if pid is None:
         raise saved_exception
 
-    # Note that we do not call update_design_documents here. This is because
+    # Note that we do not call update_design_documents and 
+    # update_pairing_service here. This is because
     # Couch won't actually have started yet, so when update_design_documents
     # calls the Records API, that will call back into get_port and we end up
     # starting Couch again. Instead, get_port calls update_design_documents
