@@ -25,7 +25,7 @@ import subprocess, sys
 from desktopcouch import local_files
 
 def stop_couchdb(ctx=local_files.DEFAULT_CONTEXT):
-    local_exec = ctx.couch_exec_command + ["-k"]
+    local_exec = ctx.couch_exec_command + ["-d"]
     try:
         retcode = subprocess.call(local_exec, shell=False)
         if retcode < 0:
