@@ -1,7 +1,7 @@
 """testing desktopcouch/start_local_couchdb.py module"""
 
 import testtools
-import os, sys
+import os, sys, shutil
 import desktopcouch.tests as test_environment
 import desktopcouch
 sys.path.append(
@@ -74,9 +74,10 @@ class TestUpdateDesignDocuments(testtools.TestCase):
         super(TestUpdateDesignDocuments, self).setUp()
         xdg_data = os.path.split(test_environment.test_context.db_dir)[0]
         try:
-            os.mkdir(os.path.join(xdg_data, "desktop-couch"))
+            shutil.rmtree(os.path.join(xdg_data, "desktop-couch"))
         except OSError:
-            pass # don't worry if the folder already exists
+            pass # don't worry if folder does not exist
+        os.mkdir(os.path.join(xdg_data, "desktop-couch"))
         for d in DIRS:
             os.mkdir(os.path.join(xdg_data, "desktop-couch", d))
         for f, data in FILES.items():
@@ -136,4 +137,43 @@ class TestUpdateDesignDocuments(testtools.TestCase):
         mocker.restore()
         mocker.verify()
 
+    def test_pair_ubuntuone_with_sso(self):
+        """ Test the pairing with ubuntu one.
+        
+        Ensure that the pair ubuntu one method is called when we do have
+        credentials.
+        """
+        from desktopcouch.start_local_couchdb import update_pairing_service
+        mocker = Mocker()
+        sso_mock = mocker.mock()
+        pair_ubuntu_one_mock = mocker.replace(
+            'desktopcouch.pair.couchdb_pairing.' + 
+            'ubuntuone_pairing.pair_with_ubuntuone')
+        # make the sso credentials return a dictionary with fake data
+        sso_mock.find_credentials('Ubuntu One')
+        mocker.result({'fake':'credential'})
+        pair_ubuntu_one_mock()
+        mocker.replay()
+        # call the start method to ensure that the methods are correctly called
+        update_pairing_service(sso_mock)
+        mocker.verify()
+
+    def test_pair_ubuntuone_without_sso(self):
+        """ Test the pairing with ubuntu one.
+        
+        Ensure that the pair ubuntu one method is called when we do have
+        credentials.
+        """
+        from desktopcouch.start_local_couchdb import update_pairing_service
+        mocker = Mocker()
+        pair_ubuntu_one_mock = mocker.replace(
+            'desktopcouch.pair.couchdb_pairing.' + 
+            'ubuntuone_pairing.pair_with_ubuntuone')
+        # make the sso credentials return a dictionary with fake data
+        mocker.result({'fake':'credential'})
+        # pair with ubuntu one never called!
+        mocker.replay()
+        # call the start method to ensure that the methods are correctly called
+        update_pairing_service(sso=None, test_import_fails=True)
+        mocker.verify()
 
